<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://mapster.me
 * @since      1.0.0
 *
 * @package    Mapster_Wordpress_Maps
 * @subpackage Mapster_Wordpress_Maps/public
 */
/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Mapster_Wordpress_Maps
 * @subpackage Mapster_Wordpress_Maps/public
 * @author     Mapster Technology Inc <hello@mapster.me>
 */
class Mapster_Wordpress_Maps_Public
{
    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private  $plugin_name ;
    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private  $version ;
    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version )
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }
    
    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {
        wp_enqueue_style( 'dashicons' );
        wp_register_style(
            'mapster_map_mapbox_css',
            plugin_dir_url( __FILE__ ) . "../admin/css/vendor/mapbox-gl-2.9.1.css",
            array(),
            $this->version
        );
        wp_register_style(
            'mapster_map_maplibre_css',
            plugin_dir_url( __FILE__ ) . "../admin/css/vendor/maplibre-1.15.2.css",
            array(),
            $this->version
        );
        wp_register_style(
            'mapster_map_directions_css',
            plugin_dir_url( __FILE__ ) . "../admin/css/vendor/directions.css",
            array(),
            $this->version
        );
        wp_register_style(
            'mapster_map_geocoder_css',
            plugin_dir_url( __FILE__ ) . "../admin/css/vendor/mapbox-gl-geocoder-4.7.2.css",
            array(),
            $this->version
        );
        wp_register_style(
            'mapster_map_geosearch_css',
            plugin_dir_url( __FILE__ ) . "../admin/css/vendor/leaflet-geosearch-3.0.5.css",
            array(),
            $this->version
        );
        wp_register_style(
            $this->plugin_name,
            plugin_dir_url( __FILE__ ) . '../admin/css/mapster-wordpress-maps.css',
            array(),
            $this->version,
            'all'
        );
        wp_register_style(
            'mapster_map_public_css',
            plugin_dir_url( __FILE__ ) . 'css/mapster-wordpress-maps-public.css',
            array(),
            $this->version,
            'all'
        );
        wp_register_style(
            'mapster_map_draw_css',
            plugin_dir_url( __FILE__ ) . "../admin/css/vendor/mapbox-gl-draw-1.3.0.css",
            array(),
            $this->version
        );
        wp_register_style(
            'mapster_map_threebox_css',
            plugin_dir_url( __FILE__ ) . "../admin/css/vendor/threebox.css",
            array(),
            $this->version
        );
    }
    
    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {
    }
    
    /**
     * Register shortcode
     *
     * @since    1.0.0
     */
    public function mapster_wordpress_maps_register_shortcodes()
    {
        add_shortcode( 'mapster_wp_map', array( $this, 'mapster_wordpress_maps_shortcode_display' ) );
    }
    
    /**
     * Add shortcode to Map type content
     *
     * @since    1.0.0
     */
    public function mapster_wordpress_maps_output_shortcode( $content )
    {
        
        if ( is_singular( 'mapster-wp-map' ) ) {
            $output_shortcode = do_shortcode( '[mapster_wp_map id="' . get_the_ID() . '"]' );
            $output_shortcode .= $content;
            return $output_shortcode;
        } else {
            
            if ( is_singular( 'mapster-wp-location' ) || is_singular( 'mapster-wp-line' ) || is_singular( 'mapster-wp-polygon' ) ) {
                
                if ( get_field( 'field_626492dbed193', get_the_ID(), true ) && get_field( 'field_6264930aed194', get_the_ID(), true ) ) {
                    $output_shortcode = do_shortcode( '[mapster_wp_map id="' . get_field( 'field_6264930aed194', get_the_ID(), true ) . '" single_feature_id="' . get_the_ID() . '"]' );
                    $output_shortcode .= $content;
                    return $output_shortcode;
                } else {
                    return $content;
                }
            
            } else {
                return $content;
            }
        
        }
    
    }
    
    /**
     * Map shortcode logic
     *
     * @since    1.0.0
     */
    public function mapster_wordpress_maps_shortcode_display( $atts )
    {
        // Handle script loading
        $settings_page_id = get_option( 'mapster_settings_page' );
        $access_token = get_field( 'default_access_token', $settings_page_id );
        $i8ln = new Mapster_Wordpress_Maps_i18n();
        $injectedParams = array(
            'strings'             => $i8ln->get_mapster_strings()['admin_js'],
            'public'              => true,
            'activated'           => ( mwm_fs()->can_use_premium_code() ? '1' : '0' ),
            'rest_url'            => get_rest_url(),
            'directory'           => plugin_dir_url( __FILE__ ),
            'mapbox_access_token' => $access_token,
        );
        $map_provider = get_field( 'map_type', $atts['id'] )['map_provider'];
        $model_3d_library = get_field( 'load_3d_model_libraries', $atts['id'] );
        // Check for required dependencies
        $directions_enabled = ( get_field( 'directions_control', $atts['id'] ) ? get_field( 'directions_control', $atts['id'] )['enable'] : false );
        $geocoder_enabled = false;
        if ( get_field( 'geocoder_control', $atts['id'] )['enable'] == true ) {
            $geocoder_enabled = true;
        }
        if ( get_field( 'filter', $atts['id'] )['custom_search_filter']['enable'] == true ) {
            $geocoder_enabled = true;
        }
        if ( get_field( 'filter', $atts['id'] )['filter_dropdown']['enable'] == true ) {
            $geocoder_enabled = true;
        }
        $last_dependency = 'jquery';
        
        if ( $map_provider === 'maplibre' ) {
            wp_enqueue_script(
                'mapster_map_' . $map_provider,
                plugin_dir_url( __FILE__ ) . "../admin/js/vendor/maplibre-1.15.2.js",
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = 'mapster_map_' . $map_provider;
        }
        
        
        if ( $map_provider === 'mapbox' ) {
            wp_enqueue_script(
                'mapster_map_' . $map_provider,
                plugin_dir_url( __FILE__ ) . "../admin/js/vendor/mapbox-gl-2.9.1.js",
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = 'mapster_map_' . $map_provider;
        }
        
        
        if ( $map_provider === 'google-maps' ) {
            $google_api_key = get_field( 'google_maps_api_key', $settings_page_id );
            wp_enqueue_script(
                'mapster_map_' . $map_provider,
                "https://maps.googleapis.com/maps/api/js?key=" . $google_api_key . "&libraries=places",
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = 'mapster_map_' . $map_provider;
        }
        
        wp_enqueue_script(
            'mapster_map_turf',
            plugin_dir_url( __FILE__ ) . "../admin/js/vendor/custom-turf.js",
            array( $last_dependency ),
            $this->version
        );
        $last_dependency = 'mapster_map_turf';
        
        if ( $directions_enabled ) {
            wp_enqueue_style( "mapster_map_directions_css" );
            wp_enqueue_script(
                'mapster_map_directions_js',
                plugin_dir_url( __FILE__ ) . "../admin/js/vendor/mapbox-gl-directions-4.1.0.js",
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = 'mapster_map_directions_js';
        }
        
        
        if ( $geocoder_enabled ) {
            wp_enqueue_style( "mapster_map_geocoder_css" );
            wp_enqueue_script(
                'mapster_map_geocoder_js',
                plugin_dir_url( __FILE__ ) . "../admin/js/vendor/mapbox-gl-geocoder-4.7.2.js",
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = 'mapster_map_geocoder_js';
        }
        
        
        if ( $model_3d_library ) {
            wp_enqueue_style( "mapster_map_threebox_css" );
            wp_enqueue_script(
                'mapster_map_threebox_js',
                plugin_dir_url( __FILE__ ) . "../admin/js/vendor/threebox.min.js",
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = 'mapster_map_threebox_js';
        }
        
        
        if ( MAPSTER_LOCAL_TESTING ) {
            wp_enqueue_script(
                $this->plugin_name . "-DownloadControl",
                plugin_dir_url( __FILE__ ) . '../admin/js/dev/controls/DownloadControl.js',
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = $this->plugin_name . "-DownloadControl";
            wp_enqueue_script(
                $this->plugin_name . "-CategoryControl",
                plugin_dir_url( __FILE__ ) . '../admin/js/dev/controls/CategoryControl.js',
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = $this->plugin_name . "-CategoryControl";
            wp_enqueue_script(
                $this->plugin_name . "-ListControl",
                plugin_dir_url( __FILE__ ) . '../admin/js/dev/controls/ListControl.js',
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = $this->plugin_name . "-ListControl";
            wp_enqueue_script(
                $this->plugin_name . "-PitchToggle",
                plugin_dir_url( __FILE__ ) . '../admin/js/dev/controls/PitchToggle.js',
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = $this->plugin_name . "-PitchToggle";
            wp_enqueue_script(
                $this->plugin_name . "-constants",
                plugin_dir_url( __FILE__ ) . '../admin/js/dev/MapsterConstants.js',
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = $this->plugin_name . "-constants";
            wp_enqueue_script(
                $this->plugin_name . "-helpers",
                plugin_dir_url( __FILE__ ) . '../admin/js/dev/MapsterHelpers.js',
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = $this->plugin_name . "-helpers";
            wp_enqueue_script(
                $this->plugin_name . "-core",
                plugin_dir_url( __FILE__ ) . '../admin/js/dev/MapsterCore.js',
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = $this->plugin_name . "-core";
            wp_enqueue_script(
                $this->plugin_name . "-container",
                plugin_dir_url( __FILE__ ) . '../admin/js/dev/MapsterContainer.js',
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = $this->plugin_name . "-container";
            wp_enqueue_script(
                $this->plugin_name . "-map",
                plugin_dir_url( __FILE__ ) . '../admin/js/dev/MapsterMap.js',
                array( $last_dependency ),
                $this->version
            );
            $last_dependency = $this->plugin_name . "-map";
            wp_register_script(
                $this->plugin_name . "-main-js",
                plugin_dir_url( __FILE__ ) . '../admin/js/dev/MapsterLoader.js',
                array( $last_dependency ),
                $this->version,
                true
            );
            
            if ( $map_provider == 'google-maps' ) {
                wp_enqueue_script(
                    $this->plugin_name . "-google-label",
                    plugin_dir_url( __FILE__ ) . '../admin/js/vendor/google-maps-label.js',
                    array( $last_dependency ),
                    $this->version
                );
                $last_dependency = $this->plugin_name . "-google-label";
                wp_enqueue_script(
                    $this->plugin_name . "-google-clustering",
                    plugin_dir_url( __FILE__ ) . '../admin/js/vendor/google-maps-clustering.js',
                    array( $last_dependency ),
                    $this->version
                );
                $last_dependency = $this->plugin_name . "-google-clustering";
                wp_enqueue_script(
                    $this->plugin_name . "-google-category-control",
                    plugin_dir_url( __FILE__ ) . '../admin/js/dev/google/CategoryControlGoogle.js',
                    array( $last_dependency ),
                    $this->version
                );
                $last_dependency = $this->plugin_name . "-google-category-control";
                wp_enqueue_script(
                    $this->plugin_name . "-google-list-control",
                    plugin_dir_url( __FILE__ ) . '../admin/js/dev/google/ListControlGoogle.js',
                    array( $last_dependency ),
                    $this->version
                );
                $last_dependency = $this->plugin_name . "-google-list-control";
                wp_enqueue_script(
                    $this->plugin_name . "-core-google",
                    plugin_dir_url( __FILE__ ) . '../admin/js/dev/google/MapsterCoreGoogle.js',
                    array( $last_dependency ),
                    $this->version
                );
                $last_dependency = $this->plugin_name . "-core-google";
                wp_enqueue_script(
                    $this->plugin_name . "-helpers-google",
                    plugin_dir_url( __FILE__ ) . '../admin/js/dev/google/MapsterHelpersGoogle.js',
                    array( $last_dependency ),
                    $this->version
                );
                $last_dependency = $this->plugin_name . "-helpers-google";
                wp_enqueue_script(
                    $this->plugin_name . "-map-google",
                    plugin_dir_url( __FILE__ ) . '../admin/js/dev/google/MapsterMapGoogle.js',
                    array( $last_dependency ),
                    $this->version
                );
                $last_dependency = $this->plugin_name . "-map-google";
            }
        
        } else {
            wp_register_script(
                $this->plugin_name . "-main-js",
                plugin_dir_url( __FILE__ ) . '../admin/js/dist/mwp.js',
                array( $last_dependency ),
                $this->version,
                true
            );
            $last_dependency = $this->plugin_name . "-main-js";
            
            if ( $map_provider == 'google-maps' ) {
                wp_enqueue_script(
                    $this->plugin_name . "-google-label",
                    plugin_dir_url( __FILE__ ) . '../admin/js/vendor/google-maps-label.js',
                    array( $last_dependency ),
                    $this->version
                );
                $last_dependency = $this->plugin_name . "-google-label";
                wp_enqueue_script(
                    $this->plugin_name . "-google-clustering",
                    plugin_dir_url( __FILE__ ) . '../admin/js/vendor/google-maps-clustering.js',
                    array( $last_dependency ),
                    $this->version
                );
                $last_dependency = $this->plugin_name . "-google-clustering";
                wp_register_script(
                    $this->plugin_name . "-google",
                    plugin_dir_url( __FILE__ ) . '../admin/js/dist/mwp-google.js',
                    array( $last_dependency ),
                    $this->version,
                    true
                );
                $last_dependency = $this->plugin_name . "-google";
            }
            
            wp_register_script(
                $this->plugin_name,
                plugin_dir_url( __FILE__ ) . '../admin/js/dist/mwp-loader.js',
                array( $last_dependency ),
                $this->version,
                true
            );
        }
        
        wp_localize_script( $this->plugin_name . "-main-js", 'mapster_params', $injectedParams );
        wp_enqueue_script( $this->plugin_name . "-main-js" );
        if ( $map_provider == 'google-maps' ) {
            wp_enqueue_script( $this->plugin_name . "-google" );
        }
        wp_enqueue_script( $this->plugin_name );
        wp_enqueue_style( "mapster_map_" . $map_provider . "_css" );
        wp_enqueue_style( $this->plugin_name );
        wp_enqueue_style( "mapster_map_public_css" );
        $single_feature_id = "";
        if ( isset( $atts["single_feature_id"] ) ) {
            $single_feature_id = $atts["single_feature_id"];
        }
        return "\r\n\t\t\t<div class='mapster-wp-maps-container'>\r\n\t\t\t\t<div class='mapster-wp-maps'\r\n\t\t\t\t\tid='mapster-wp-maps-" . $atts['id'] . "'\r\n\t\t\t\t\tdata-id='" . $atts['id'] . "'\r\n\t\t\t\t\tdata-latitude='" . (( isset( $atts['latitude'] ) ? $atts['latitude'] : "null" )) . "'\r\n\t\t\t\t\tdata-longitude='" . (( isset( $atts['longitude'] ) ? $atts['longitude'] : "null" )) . "'\r\n\t\t\t\t\tdata-zoom='" . (( isset( $atts['zoom'] ) ? $atts['zoom'] : "null" )) . "'\r\n\t\t\t\t\tdata-single_feature_id='" . $single_feature_id . "'>\r\n\t\t\t\t</div>\r\n\t\t\t</div>\r\n\t\t";
    }

}